/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the  "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.emje.treehtmlparse.urlfilter;

import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

/**
 *
 * @author Ammar Shadiq
 */
public class AddAcceptedURLView extends javax.swing.JDialog {

    private static AddAcceptedURLView addAcceptedURLView;
    private String sourceURL;
    private static String regexTextEdit;
    private String regexTextBackup;
   
    public static String showDialog(java.awt.Frame parent, boolean modal, 
            String title, String regexText, String sourceURL)
    {
        addAcceptedURLView = new AddAcceptedURLView(parent, modal, title, regexText, sourceURL);
        addAcceptedURLView.setLocationRelativeTo(parent);
        addAcceptedURLView.setVisible(true);
        return regexTextEdit;
    }
    
    
    public AddAcceptedURLView(java.awt.Frame parent, boolean modal, String title, 
            String regexText, String sourceURL) 
    {
        super(parent, modal);
        regexTextBackup = regexText;
        this.sourceURL = sourceURL;
        
        initComponents();
        this.acceptedURLTextField.setText(regexText);
        setTitle(title);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        valueLabel = new javax.swing.JLabel();
        acceptedURLTextField = new javax.swing.JTextField();
        cancelButton = new javax.swing.JButton();
        okButton = new javax.swing.JButton();
        checkButton = new javax.swing.JButton();
        checkStatus = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setName("Form"); // NOI18N
        setResizable(false);

        valueLabel.setText(org.openide.util.NbBundle.getMessage(AddAcceptedURLView.class, "AddAcceptedURLView.valueLabel.text")); // NOI18N
        valueLabel.setName("valueLabel"); // NOI18N

        acceptedURLTextField.setText(org.openide.util.NbBundle.getMessage(AddAcceptedURLView.class, "AddAcceptedURLView.acceptedURLTextField.text")); // NOI18N
        acceptedURLTextField.setName("acceptedURLTextField"); // NOI18N
        acceptedURLTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                acceptedURLTextFieldActionPerformed(evt);
            }
        });

        cancelButton.setText(org.openide.util.NbBundle.getMessage(AddAcceptedURLView.class, "AddAcceptedURLView.cancelButton.text")); // NOI18N
        cancelButton.setName("cancelButton"); // NOI18N
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelButtonActionPerformed(evt);
            }
        });

        okButton.setText(org.openide.util.NbBundle.getMessage(AddAcceptedURLView.class, "AddAcceptedURLView.okButton.text")); // NOI18N
        okButton.setName("okButton"); // NOI18N
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okButtonActionPerformed(evt);
            }
        });

        checkButton.setText(org.openide.util.NbBundle.getMessage(AddAcceptedURLView.class, "AddAcceptedURLView.checkButton.text")); // NOI18N
        checkButton.setName("checkButton"); // NOI18N
        checkButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                checkButtonActionPerformed(evt);
            }
        });
        checkButton.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                checkButtonFocusLost(evt);
            }
        });

        checkStatus.setForeground(java.awt.Color.red);
        checkStatus.setText(org.openide.util.NbBundle.getMessage(AddAcceptedURLView.class, "AddAcceptedURLView.checkStatus.text")); // NOI18N
        checkStatus.setVisible(false);
        checkStatus.setName("checkStatus"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(valueLabel)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(acceptedURLTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 449, Short.MAX_VALUE))
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addComponent(checkButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(checkStatus)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 306, Short.MAX_VALUE)
                        .addComponent(okButton)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(cancelButton)))
                .addContainerGap())
        );

        layout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {cancelButton, okButton});

        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(valueLabel)
                    .addComponent(acceptedURLTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(cancelButton)
                    .addComponent(okButton)
                    .addComponent(checkButton)
                    .addComponent(checkStatus))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancelButtonActionPerformed
        // present the previous text string for the return value
        //close this window frame
        regexTextEdit = regexTextBackup;
        addAcceptedURLView.setVisible(false);
        addAcceptedURLView.dispose();
    }//GEN-LAST:event_cancelButtonActionPerformed

    private void okButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_okButtonActionPerformed
        // present the current text string for the return value
        regexTextEdit = this.acceptedURLTextField.getText();
        regexTextBackup = null;
        addAcceptedURLView.setVisible(false);
        addAcceptedURLView.dispose();
    }//GEN-LAST:event_okButtonActionPerformed

    private void acceptedURLTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_acceptedURLTextFieldActionPerformed
        okButtonActionPerformed(evt);
    }//GEN-LAST:event_acceptedURLTextFieldActionPerformed

    private void checkButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_checkButtonActionPerformed
        String regExPatternString = acceptedURLTextField.getText();
        try {
            Pattern pattern = Pattern.compile(regExPatternString);
            Matcher matcher = pattern.matcher(sourceURL);
            if (matcher.matches()) {
                checkStatus.setForeground(java.awt.Color.green);
                checkStatus.setText("RegEx Matches with current URL");
                checkStatus.setVisible(true);
            } else {
                checkStatus.setForeground(java.awt.Color.red);
                checkStatus.setText("RegEx not Matches with current URL");
                checkStatus.setVisible(true);
            }
        } catch (PatternSyntaxException pse) {
            checkStatus.setForeground(java.awt.Color.red);
                checkStatus.setText("RegEx not Matches with current URL");
                checkStatus.setVisible(true);
        }
    }//GEN-LAST:event_checkButtonActionPerformed

    private void checkButtonFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_checkButtonFocusLost
        checkStatus.setVisible(false);
    }//GEN-LAST:event_checkButtonFocusLost

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JTextField acceptedURLTextField;
    private javax.swing.JButton cancelButton;
    private javax.swing.JButton checkButton;
    private javax.swing.JLabel checkStatus;
    private javax.swing.JButton okButton;
    private javax.swing.JLabel valueLabel;
    // End of variables declaration//GEN-END:variables

    
}
